# Chapter 19
# Symbolic Computation of Explicit Runge-Kutta Formulas
# D. Gruntz, gruntz@fh-aargau.ch

restart;

diff(f(x, y(x)), x);

restart;
`diff/y` := (a, x) -> f(a, y(a))*diff(a, x):
diff(y(x), x);

diff(y(x), x$2);

D(y) := x -> f(x, y(x)):
taylor(y(x+h), h=0, 3);

alias(F = f(x,y(x)), 
   F[x] = D[1](f)(x,y(x)), F[y] = D[2](f)(x,y(x)),
   F[x,x] = D[1,1](f)(x,y(x)), F[x,y] = D[1,2](f)(x,y(x)), 
   F[y,y] = D[2,2](f)(x,y(x))):

diff(F, x);

diff(F, x$2);

taylor(y(x+h), h=0, 3);

taylor(f(x+h, y(x)+h), h=0, 3);

m := 3:
taylor(y(x+h), h=0, m+1);

TaylorPhi := normal((convert(%, polynom) - y(x))/h);

k[1]:=taylor(f(x,       y(x)),                            h=0,m):
k[2]:=taylor(f(x+c[2]*h,y(x)+h*(a[2,1]*k[1])),            h=0,m):
k[3]:=taylor(f(x+c[3]*h,y(x)+h*(a[3,1]*k[1]+a[3,2]*k[2])),h=0,m):
RungeKuttaPhi := series(b[1]*k[1]+b[2]*k[2]+b[3]*k[3], h, m):

RungeKuttaPhi := convert(RungeKuttaPhi, polynom);

d := expand(TaylorPhi - RungeKuttaPhi):
eqns := {coeffs(d, [h,F,F[x],F[y],F[x,x],F[x,y],F[y,y]])};

vars := indets(eqns);

sols := solve(eqns, vars);

G := grobner[gbasis](eqns, 
         [b[1],b[2],c[2],a[2,1],a[3,1],a[3,2],b[3],c[3]], plex);

a[3,2] := normal(solve(G[7], a[3,2]));

a[3,1] := normal(solve(G[6], a[3,1]));

a[2,1] := normal(solve(G[5], a[2,1]));

normal(G[4]);

c[2] := normal(solve(G[3], c[2]));

b[2] := factor(solve(G[2], b[2]));

b[1] := normal(solve(G[1], b[1]));

a := 'a': b := 'b': c := 'c':
grobner[gbasis]([G[], 3*b[3]^2*c[3]^2-b[3]],
        [b[1],b[2],c[2],a[2,1],a[3,1],a[3,2],b[3],c[3]], plex);

X := [b[1],a[3,1],b[2],c[2],a[2,1],a[3,2],b[3],c[3]]:
for i to nops(X) do B[i] := {} od:
for p in eqns do
   for i while not has(p, X[i]) do od;
   B[i] := B[i] union {primpart(p)}
od:

seq(`B[` || i || `]` = B[i], i=1..nops(X));

primpart(resultant(B[3][1], B[3][2], b[2]));

B[4] := B[4] union {%}:
primpart(resultant(B[4][1], B[4][2], c[2]));

B[6] := B[6] union {%}:

a[3,2] := normal(solve(B[6][1], a[3,2]));

a[2,1] := normal(solve(B[5][1], a[2,1]));

c[2] :=op({solve(B[4][1], c[2])} minus {0});

b[2] := normal(solve(B[3][1], b[2]));

a[3,1] := normal(solve(B[2][2], a[3,1]));

b[1] := normal(solve(B[1][1], b[1]));

restart;
read `RungeKutta.map`;
RungeKutta(2, 2);

restart;
read `RungeKutta.map`;
# the following command uses many cpu cycles...
RK4 := RungeKutta(4, 4):
# alternatively, you can also read in the solution
read `rukut.sol`:

RK4[2];

alias(alpha = RootOf(
     576*(1-c[2]+4*a[3,2]*c[2]^2-2*a[3,2]*c[2])*_Z^2 
     +24*(c[2]-2)*_Z + 1)):
RK4[1];

solve(subs(RK4[1], {a[3,1]=0, a[4,1]=0, a[4,2]=0}));

subs(%, RK4[1]);

alias(beta=RootOf(1-36*_Z+288*_Z^2)):
allvalues(beta);
subs(beta=1/12, %%%);

subs(c[2]=1/3, a[3,2]=1, RK4[1]);

alias(gamma=RootOf(1-40*_Z+256*_Z^2)):
allvalues(gamma);
subs(gamma=1/8, %%%);

